import java.lang.RuntimeException;

/**
 * La classe Terrain1D modélise un terrain à une dimension découpé en
 * cases successives. Chaque case peut contenir de l'herbe
 * (représentée par le caractère '.'), une souris ('m'), ou être vide (' ').
 * À chaque itération, la souris se déplace pour essayer de manger de
 * l'herbe, et de l'herbe peut repousser aléatoirement.
 */

class Terrain1D {
    // Attributs
    char [] terrain;                                        // À faire
    double probaHerbePousse = 0;                            // À faire

    // Méthodes
    /**
     * @param n: un entier positif
     *
     * Construit un nouveau terrain vide de longueur n
     */
    public Terrain1D(int n) {                               // À faire
        terrain = new char[n];
        for (int i=0; i<n; i++)
            terrain[i] = ' ';
    }

    /**
     * @param n: un entier positif
     * @param p: un double entre 0 et 1 inclus
     *
     * Construit un nouveau terrain de longueur n, où chaque case contient de l'herbe avec probabilité p
     */
    public Terrain1D(int n, double p) {                     // À faire
        terrain = new char[n];
        for (int i=0; i<n; i++)
            if (Math.random() <= p)
                terrain[i] = '.';
            else
                terrain[i] = ' ';
    }

    /**
     * @param s: une chaine de caractères
     *
     * Construit un nouveau terrain initialisé à partir de s
     */
    public Terrain1D(String s) {                            // À faire
        terrain = new char[s.length()];
        for (int i=0; i<s.length(); i++)
            terrain[i] = s.charAt(i);
    }

    /**
     * Ajoute une souris à une position aléatoire
     **/
    public void ajouteSouris() {                            // À faire
        int position = (int) (Math.random() * terrain.length);
        terrain[position] = 'm';
    }

    /**
     * Essaye d'ajouter de l'herbe à une position aléatoire
     *
     * S'il y a une souris à cette position, la souris la mange instantanément
     * S'il y a déjà de l'herbe à cette position, il ne se passe rien
     **/
    public void ajouteHerbe() {                             // À faire
        int position = (int) (Math.random() * terrain.length);
        if (terrain[position] == ' ')
            terrain[position] = '.';
    }

    /**
     * @return: une chaine de caractères
     *
     * Renvoie une chaîne de caractère décrivant le contenu du terrain
     */
    public String toString() {                              // À faire
        return new String(terrain);
    }

    /**
     * @param c: un caractère
     *
     * @return le nombre d'occurences de `c` sur le terrain.
     **/
    public int nombreOccurences(char c) {                   // À faire
        int compteur = 0;
        for (int i=0; i<terrain.length; i++) {
            if (terrain[i] == c)
                compteur++;
        }
        return compteur;
    }

    /**
     * @param c: un caractère
     *
     * @return la position de la première occurence de `c` sur le
     * terrain. Si `c` n’apparaît pas dans le tableau, renvoie −1.
     *
     * Application typique: chercher la souris
     **/
    public int position(char c) {                           // À faire
        for (int i=0; i<terrain.length; i++) {
            if (terrain[i] == c)
                return i;
        }
        return -1;
    }

    /**
     * @param position: un entier
     * @param c: un caractère
     *
     * @return un entier `i` le plus proche possible de `position` tel que
     * `tableau[i] == c`. Si `c` n’apparaît pas dans le tableau, renvoie
     * −1.
     *
     * Application typique: connaissant la position de la souris,
     * chercher l'herbe la plus proche
     **/
    public int plusProchePosition(int position, char c) {   // À faire
        for (int i = position, j = position; i>=0 || j < terrain.length; i--, j++) {
            if (i >=0              && terrain[i] == c)
                return i;
            if (j < terrain.length && terrain[j] == c)
                return j;
        }
        return -1;
    }

    /**
     * @param p un double entre 0 et 1 inclus
     *
     * Configure la probabilité que de l'herbe essaye de pousser à chaque itération
     **/
    public void setProbaHerbePousse(double p) {             // À faire
        probaHerbePousse = p;
    }

    /**
     * @precondition il y a exactement une souris sur le terrain
     *
     * Avec probabilité probaHerbePousse, essaye de rajouter de l'herbe.
     * (voir setProbaHerbePousse et ajouteHerbe)
     *
     * S'il y a une souris, la déplace d'une position vers l'herbe la plus proche.
     * S'il n'y a plus d'herbe, la souris meure.
     **/
    public void evolue() {
        if (Math.random() <= probaHerbePousse) {            // À faire
            ajouteHerbe();
        }
        int positionSouris = position('m');
        if (positionSouris == -1) {
            // Pas de souris
	    return;
        }
        int positionHerbe = plusProchePosition(positionSouris, '.');
        if (positionHerbe == -1) {
            // La souris meure
            terrain[positionSouris] = ' ';
	    return;
        }
        if (positionHerbe > positionSouris) {
            terrain[positionSouris+1] = 'm';
            terrain[positionSouris]   = ' ';
        }
        if (positionHerbe < positionSouris) {
            terrain[positionSouris-1] = 'm';
            terrain[positionSouris]   = ' ';
        }
    }
}
